// StopLightDisplay class for CSCI 201 StopLight lab

// (c) J. Dean Brock, 2004, 2006
// Department of Computer Science
// UNC Asheville

// Based on C++ StopLight
// (c) J. Dean Brock, 1998

package edu.unca.cs.csci201.labaids;

import java.awt.* ;
import java.awt.geom.* ;

class StopLightDisplay extends javax.swing.JComponent {
    
    // Size of entire display
    private Dimension saveSize  = new Dimension() ;
    // Size and position of stoplight "box" within the display
    private Dimension sizeBox   = new Dimension() ;
    private Point     cornerBox = new Point() ;
    // Size of each light bulb
    private Dimension sizeLight = new Dimension() ;
    // Position of each bulb
    private Point  cornerRed    = new Point() ;
    private Point  cornerYellow = new Point() ;
    private Point  cornerGreen  = new Point() ;

    // Colors
    private static final Color colorRedOn     = new Color(255,   0, 0) ;
    private static final Color colorRedOff    = new Color( 75,   0, 0) ;
    private static final Color colorYellowOn  = new Color(255, 255, 0) ;
    private static final Color colorYellowOff = new Color( 75,  75, 0) ;
    private static final Color colorGreenOn   = new Color(  0, 255, 0) ;
    private static final Color colorGreenOff  = new Color(  0,  75, 0) ;
    
    // Satte of the light
    private boolean stateRed    = false ;
    private boolean stateYellow = false ;
    private boolean stateGreen  = false ;

    private void ComputeBulbPosition() {
        // Get the size of the display area given to us
        saveSize.width  = getWidth() ;
        saveSize.height = getHeight() ;
        if (saveSize.height > 3*saveSize.width)
            sizeBox.width  = saveSize.width ;
        else
            sizeBox.width  = saveSize.height/3 ;
        sizeBox.height = 3*sizeBox.width ;
        
        cornerBox.x = (saveSize.width-sizeBox.width)/2 ;
        cornerBox.y = (saveSize.height-sizeBox.height)/2 ;
        
        sizeLight.width  = (sizeBox.width*4+2)/5 ;
        sizeLight.height = sizeLight.width ;

        cornerRed.x = cornerBox.x + (sizeBox.width+5)/10 ;
        cornerRed.y = cornerBox.y + (sizeBox.width+5)/10 ;
        
        cornerYellow.x = cornerRed.x ;
        cornerYellow.y = cornerBox.y + (11*sizeBox.width+5)/10 ;
        
        cornerGreen.x = cornerRed.x ;
        cornerGreen.y = cornerBox.y + (21*sizeBox.width+5)/10 ;
    }
    
    public void paint(Graphics g) {
        Graphics ng = g.create() ;
        if (getWidth() != saveSize.width || getHeight() != saveSize.height) {
            // super.paint(g) ;
            ComputeBulbPosition() ;
        }
        
        ng.setColor(Color.BLACK) ;
        ng.fillRect(cornerBox.x, cornerBox.y, sizeBox.width, sizeBox.height) ;
        
        if (stateRed)
            setLightColor(ng, colorRedOn, cornerRed.x, cornerRed.y) ;
        else
            setLightColor(ng, colorRedOff, cornerRed.x, cornerRed.y) ;
        
        if (stateYellow)
            setLightColor(ng, colorYellowOn, cornerYellow.x, cornerYellow.y) ;
        else
            setLightColor(ng, colorYellowOff, cornerYellow.x, cornerYellow.y) ;
        
        if (stateGreen)
            setLightColor(ng, colorGreenOn, cornerGreen.x, cornerGreen.y) ;
        else
            setLightColor(ng, colorGreenOff, cornerGreen.x, cornerGreen.y) ;
    }

    private void setLightColor(Graphics g, Color c, int cx, int cy) {
        // if (g!=null)
	g.setColor(c) ;
        g.fillOval(cx, cy, sizeLight.width, sizeLight.height) ;
    }
    
    public void setRed(boolean newRed) {
        stateRed = newRed ;
        repaint() ;
        // setLightColor(getGraphics(), colorRedOn, cornerRed.x, cornerRed.y) ;
    }
    
    public boolean getRed() {
        return stateRed ;
    }
    
    public void setGreen(boolean newGreen) {
        stateGreen = newGreen ;
        repaint() ;
    }
    
    public boolean getGreen() {
        return stateGreen ;
    }
    
    public void setYellow(boolean newYellow) {
        stateYellow = newYellow ;
        repaint() ;
    }
    
    public boolean getYellow() {
        return stateYellow ;
    }
}
